import 'package:async/async.dart';
import 'package:flutter/widgets.dart';
import 'package:leancode_common/leancode_common.dart';

class AsyncInitializer<T extends AsyncInitializable> extends StatefulWidget {
  const AsyncInitializer({
    @required this.model,
    this.child,
    this.onLoading,
    Key key,
  })  : assert(model != null),
        super(key: key);

  final T model;
  final Widget onLoading;
  final Widget child;

  @override
  _AsyncInitializerState createState() => _AsyncInitializerState(model);
}

class _AsyncInitializerState extends State<AsyncInitializer> {
  _AsyncInitializerState(this._model) : _memoizer = AsyncMemoizer();

  AsyncMemoizer<void> _memoizer;
  AsyncInitializable _model;

  @override
  void didUpdateWidget(AsyncInitializer<AsyncInitializable> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.model != widget.model) {
      _model = widget.model;
      _memoizer = AsyncMemoizer();
    }
  }

  @override
  Widget build(BuildContext context) {
    return _model is AsyncInitializable
        ? FutureBuilder<void>(
            future: _memoizer.runOnce(_model.initializeAsync),
            builder: (ctx, snapshot) {
              return snapshot.connectionState == ConnectionState.done
                  ? widget.child
                  : widget.onLoading;
            })
        : widget.child;
  }
}
