import 'dart:async';

import 'package:rxdart/rxdart.dart';

class ValuePublishSubject<T> extends Subject<T> implements ValueObservable<T> {
  factory ValuePublishSubject(
      {void onListen(), void onCancel(), bool sync = false, T seedValue}) {
    final controller = StreamController<T>.broadcast(
      onListen: onListen,
      onCancel: onCancel,
      sync: sync,
    );

    return ValuePublishSubject<T>._(
      controller,
      Observable<T>(controller.stream),
      seedValue,
    );
  }

  ValuePublishSubject._(StreamController<T> controller,
      Observable<T> observable, this._latestValue)
      : _latestIsValue = _latestValue != null,
        super(controller, observable);

  // Use it as initialData in StreamBuilder
  T get value => _latestValue;

  T _latestValue;
  bool _latestIsValue;

  bool get hasValue => _latestIsValue;

  @override
  void onAdd(T event) {
    _latestValue = event;
    _latestIsValue = true;

    super.onAdd(event);
  }

  @override
  void onAddError(Object error, [StackTrace stackTrace]) {
    _latestIsValue = false;
    _latestValue = null;

    super.onAddError(error, stackTrace);
  }
}
