import 'dart:async';

import 'package:test_api/test_api.dart';

import '../lib/leancode_common_flutter.dart';

void main() {
  group('Value as initialData', () {
    test('start with null value', () {
      final controller = ValuePublishSubject<int>();
      expect(controller.value, null);
      expect(controller.hasValue, false);

      controller.close();
      expect(controller.isClosed, true);
    });

    test('start with any value', () {
      final controller = ValuePublishSubject<int>(seedValue: 5);
      expect(controller.value, 5);
      expect(controller.hasValue, true);

      controller.close();
    });
  });

  test('Events addition', () {
    final controller = ValuePublishSubject<int>();

    expect(controller.value, null);

    controller.add(3);
    expect(controller.value, 3);
    expect(controller.hasValue, true);

    controller.add(5);
    expect(controller.value, 5);
    expect(controller.hasValue, true);

    controller.addError('error');
    expect(controller.value, null);
    expect(controller.hasValue, false);

    controller.add(7);
    expect(controller.value, 7);
    expect(controller.hasValue, true);

    controller.close();
  });

  test('Do not distribute initial data ', () async {
    final subject = ValuePublishSubject<int>(seedValue: 2);

    scheduleMicrotask(() {
      subject.close();
    });

    await expectLater(subject, emits(emitsDone));
  });

  test('Listen on events', () async {
    final controller = ValuePublishSubject<int>();

    scheduleMicrotask(() {
      controller.add(1);
      controller.add(5);
      controller.add(9);
      controller.close();
    });

    await expectLater(controller, emitsInOrder(<dynamic>[1, 5, 9, emitsDone]));
  });
}
